@@ -16,10 +16,10 @@ logger = b4.logger
 def cmd_retrieval_common_opts(sp):
     sp.add_argument('msgid', nargs='?',
                     help='Message ID to process, or pipe a raw message')
-    sp.add_argument('-p', '--use-project', dest='useproject', default=None,
-                    help='Use a specific project instead of default (linux-mm, linux-hardening, etc)')
     sp.add_argument('-m', '--use-local-mbox', dest='localmbox', default=None,
                     help='Instead of grabbing a thread from lore, process this mbox file (or - for stdin)')
+    sp.add_argument('--stdin-pipe-sep',
+                    help='When accepting messages on stdin, split using this pipe separator string')
     sp.add_argument('-C', '--no-cache', dest='nocache', action='store_true', default=False,
                     help='Do not use local cache')
 
@@ -111,7 +111,7 @@ def cmd_diff(cmdargs):
     b4.diff.main(cmdargs)
 
 
-def cmd():
+def setup_parser() -> argparse.ArgumentParser:
     # noinspection PyTypeChecker
     parser = argparse.ArgumentParser(
         prog='b4',
@@ -126,6 +126,10 @@ def cmd():
                         help='Output critical information only')
     parser.add_argument('-n', '--no-interactive', action='store_true', default=False,
                         help='Do not ask any interactive questions')
+    parser.add_argument('--offline-mode', action='store_true', default=False,
+                        help='Do not perform any network queries')
+    parser.add_argument('--no-stdin', action='store_true', default=False,
+                        help='Disable TTY detection for stdin')
 
     subparsers = parser.add_subparsers(help='sub-command help', dest='subcmd')
 
@@ -228,8 +232,6 @@ def cmd():
                          help='Message ID to process, or pipe a raw message')
     sp_diff.add_argument('-g', '--gitdir', default=None,
                          help='Operate on this git tree instead of current dir')
-    sp_diff.add_argument('-p', '--use-project', dest='useproject', default=None,
-                         help='Use a specific project instead of default (linux-mm, linux-hardening, etc)')
     sp_diff.add_argument('-C', '--no-cache', dest='nocache', action='store_true', default=False,
                          help='Do not use local cache')
     sp_diff.add_argument('-v', '--compare-versions', dest='wantvers', type=int, default=None, nargs='+',
@@ -264,8 +266,15 @@ def cmd():
                        help='Show current series revision number')
     spp_g.add_argument('--force-revision', metavar='N', type=int,
                        help='Force revision to be this number instead')
+    spp_g.add_argument('--compare-to', metavar='vN',
+                       help='Display a range-diff to previously sent revision N')
     spp_g.add_argument('--manual-reroll', dest='reroll', default=None, metavar='COVER_MSGID',
                        help='Mark current revision as sent and reroll (requires cover letter msgid)')
+    spp_g.add_argument('--set-prefixes', metavar='PREFIX', nargs='+',
+                       help='Extra prefixes to add to [PATCH] (e.g.: RFC mydrv)')
+    spp_g.add_argument('--show-info', action='store_true', default=False,
+                       help='Show current series info in a column-parseable format')
+
     ag_prepn = sp_prep.add_argument_group('Create new branch', 'Create a new branch for working on patch series')
     ag_prepn.add_argument('-n', '--new', dest='new_series_name',
                           help='Create a new branch for working on a patch series')
@@ -284,10 +293,11 @@ def cmd():
                         help='Update branch commits with latest received trailers')
     sp_trl.add_argument('-S', '--sloppy-trailers', dest='sloppytrailers', action='store_true', default=False,
                         help='Apply trailers without email address match checking')
-    sp_trl.add_argument('-F', '--trailers-from', dest='msgid',
+    sp_trl.add_argument('-F', '--trailers-from', dest='trailers_from',
                         help='Look for trailers in the thread with this msgid instead of using the series change-id')
     sp_trl.add_argument('--since', default='1.month',
                         help='The --since option to use with -F when auto-matching patches (default=1.month)')
+    cmd_retrieval_common_opts(sp_trl)
     sp_trl.set_defaults(func=cmd_trailers)
 
     # b4 send
@@ -296,8 +306,8 @@ def cmd():
                          help='Do not send, just dump out raw smtp messages to the stdout')
     sp_send.add_argument('-o', '--output-dir',
                          help='Do not send, write raw messages to this directory (forces --dry-run)')
-    sp_send.add_argument('--prefixes', nargs='+',
-                         help='Prefixes to add to PATCH (e.g. RFC, WIP)')
+    sp_send.add_argument('--reflect', action='store_true', default=False,
+                         help='Send everything to yourself instead of the actual recipients')
     sp_send.add_argument('--no-trailer-to-cc', action='store_true', default=False,
                          help='Do not add any addresses found in the cover or patch trailers to To: or Cc:')
     sp_send.add_argument('--hide-cover-to-cc', action='store_true', default=False,
@@ -306,7 +316,7 @@ def cmd():
     sp_send.add_argument('--cc', nargs='+', help='Addresses to add to the Cc: list')
     sp_send.add_argument('--not-me-too', action='store_true', default=False,
                          help='Remove yourself from the To: or Cc: list')
-    sp_send.add_argument('--resend', default=None,
+    sp_send.add_argument('--resend', metavar='vN', default=None,
                          help='Resend a previously sent version of the series')
     sp_send.add_argument('--no-sign', action='store_true', default=False,
                          help='Do not add the cryptographic attestation signature header')
@@ -317,8 +327,12 @@ def cmd():
                           help='Submit the token received via verification email')
     sp_send.set_defaults(func=cmd_send)
 
-    cmdargs = parser.parse_args()
+    return parser
 
+
+def cmd():
+    parser = setup_parser()
+    cmdargs = parser.parse_args()
     logger.setLevel(logging.DEBUG)
 
     ch = logging.StreamHandler()
@@ -338,6 +352,10 @@ def cmd():
         parser.print_help()
         sys.exit(1)
 
+    if cmdargs.offline_mode:
+        logger.info('Running in OFFLINE mode')
+        b4.can_network = False
+
     cmdargs.func(cmdargs)
 
 
